Unit IBMDAA;
interface
  {no other units used }

  {The Card number is set during initialization by InitIBM. Initializing
   sets Waveset to false; in order to synchronize with the square wave
   timer output, it's necessary to do the Timebase procedure after every
   initialization. Readtime returns a number that counts down toward zero
   over and over. Timer 0 puts out a square wave with a period that is twice
   as long as the argument of TimeBaseIBM. For a total period of 1 msec, the
   argument should be 2046.

   Twentymike is set to 50, and is used for a delay after selecting the
   channel with AnPickIBM. This value should be OK for 10 MHz AT computers.}


Procedure InitIBM(Cardnum: word);            { initialize card, 0..3 }
Procedure AnPickIBM(channel:word);           {Select a/d converter }
Function AnReadIBM(Channel: word): Integer;    { read channel }
Procedure AnWriteIBM(Channel,data: word);      { write channel }
Function BinReadIBM: word;                     { read 16 bits }
Procedure BinWriteIBM(data: word);             { write 16 bits }
procedure TimebaseIBM(period: word);   { start square wave from timer 0 }
function ReadTimeIBM: Word;               { read count-down register, timer 0 }

var twentymike: integer;   {to be adjusted for processor speed }


implementation

type
  Str80       = string[80];

Const
  DevReg     = $C000;
  IntrReg    = $D000;
  AnaDev     = 9;
  BinDev     = 8;
  AIContReg  = 0;
  AIStatReg  = 0;
  AIReadReg  = $2000;
  HiByte     = 1;
  LoByte     = 0;
  AOContReg  = $1000;
  AOWriteReg = $3000;
  BinRWReg   = $2000;
  ADOff      = 0;
  ADOn       = 1;
  Time0      = 0;
  Time1      = 1;
  Time2      = 2;
  TContReg   = $1000 * 11;
  Time2Reg   = $1000 * 10;
  Time1Reg   = $1000 * 9;
  Time0Reg   = $1000 * 8;

Var
  BaseAddr,RegAddr,card: word;
  Waveset: boolean;


{ ***************************************************************************}

Procedure InitIBM(Cardnum: word); { Card number to initialize }
var i: integer;
begin
 twentymike := 50;                { default for 20 microsecond delay }
 Card := cardnum AND 3;            { force card number to 0 .. 3 }
 Waveset := false;                 { must start wave before synch works }
 BaseAddr := Card * $400 + $2e2;
 RegAddr := BaseAddr + $400 * Card;
 Port[BaseAddr+IntrReg] := 0;   { disable all interrupts }
 for i := 0 to twentymike do ;
end; { InitIBMAD }

{ ***************************************************************************}
procedure AnPickIBM(Channel: word);
var i: integer;
begin
 Port[BaseAddr + DevReg] := AnaDev;             { select analogue device }
 Port[RegAddr + AIContReg] := ADOff;
 Port[RegAddr + AIContReg + HiByte] := Channel;      { Set Channel }
 for i := 0 to twentymike do ;
end;

{ ***************************************************************************}

Function AnReadIBM(Channel: word): integer;
var temp1,temp2: integer;
begin
 Port[BaseAddr + DevReg] := AnaDev;             { select analogue device }
 Port[RegAddr + AIContReg] := ADOn;
 Port[RegAddr + AIContReg + HiByte] := Channel;       { start conversion }
 while (Port[RegAddr + AIContReg] AND 1) = 1 do ; { wait for not busy }
 Port[RegAddr + AIContReg] := ADoff;
 Port[RegAddr + AIContReg + HiByte] :=  Channel;     { prepare to read }
 temp1 := Port[RegAddr + AIReadReg];
 temp2 := Port[RegAddr + AIReadReg + HiByte];
 AnReadIBM := temp1 + 256 * temp2;
end; { AnReadIBM }

{ ***************************************************************************}

Procedure AnWriteIBM(Channel,data: word);
begin
 Port[BaseAddr + DevReg] := AnaDev;      { select analogue device }
 Port[RegAddr + AOContReg + HiByte] := (channel and 1); { Set chan, force 0..1 }
 Portw[RegAddr + AOWriteReg] := data; { output data }
end; { WriteIBMAD }

{ ***************************************************************************}

function BinReadIBM: word;
begin
 Port[BaseAddr + DevReg] := BinDev;      { select binary device }
 BinReadIBM := Portw[RegAddr + BinRWReg];
end;

{ ***************************************************************************}

procedure BinWriteIBM(data: word);
begin
 Port[BaseAddr + DevReg] := BinDev;      { select binary device }
 Portw[RegAddr + BinRWReg] := data;        { output data }
end;

{ ***************************************************************************}

procedure TimebaseIBM(period: word);   {square wave from Rate, timer 0 }
begin
 Port[BaseAddr + DevReg] := Time0;   { select Timer 0 }
 Port[RegAddr + TContReg] := $36;     { Counter 0, Seq load, Mode 3, Binary }
 Port[RegAddr + Time0Reg] := (period and $ff);          { load low byte }
 Port[RegAddr + Time0Reg] := (period and $ff00) shr 8;  { load high byte }
 waveset := true;
end;

{ ***************************************************************************}

function ReadTimeIBM: Word;
var j,k: word;
begin
  Port[RegAddr + TContReg] := $06;     { Latch Timer 0 }
  j := Port[RegAddr + Time0Reg];        { read lsb }
  k := Port[RegAddr + Time0Reg];        { read msb }
  ReadTimeIBM := j + k shl 8;    { make into word and return it }
end;


{ ***************************************************************************}

end. { IBMDAA unit }
